@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2017 The OpenMx Project
+#   Copyright 2007-2018 The OpenMx Project
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -126,7 +126,7 @@ if(mxOption(NULL,"Default optimizer")=="NPSOL"){
 		c("x1.bound","x2.bound","x3.bound","x4.bound","x5.bound","PowellBenchmarkWithJacobians.c1[1,1]",
 			"PowellBenchmarkWithJacobians.c2[1,1]","PowellBenchmarkWithJacobians.c3[1,1]")
 	)
-} else if(mxOption(NULL,"Default optimizer")=="SLSQP"){
+} else if(mxOption(NULL,"Default optimizer") %in% c("CSOLNP","SLSQP")){
 	#Analytic Jacobians should, if nothing else, cut down on the number of fitfunction evaluations:
 	omxCheckEquals(omxGreaterThan(powellrun1$output$evaluations,powellrun2$output$evaluations),1)
 	
@@ -261,7 +261,7 @@ powellrun4 <- mxRun(powellmod4)
 omxCheckCloseEnough(powellrun3$fitfunction$result, powellrun4$fitfunction$result,1e-7)
 powellrun4$output$iterations
 powellrun4$output$evaluations
-omxCheckCloseEnough(coef(powellrun3),coef(powellrun4),1e-7)
+omxCheckCloseEnough(coef(powellrun3),coef(powellrun4),1e-6)
 summary(powellrun4)
 
 
@@ -368,17 +368,14 @@ powellrun7$output$iterations
 powellrun7$output$evaluations
 summary(powellrun7)
 
-if(mxOption(NULL,"Default optimizer") %in% c("NPSOL","SLSQP")){
-	tbl <- data.frame(
-		c("Yes","Yes","No","No"),c("No","Yes","Yes","No"),
-		c(powellrun1$output$evaluations,powellrun2$output$evaluations,powellrun6$output$evaluations,
-			powellrun7$output$evaluations), row.names=c("powellrun1","powellrun2","powellrun6","powellrun7"),
-		stringsAsFactors=F
-		)
-	colnames(tbl) <- c("Gradient?","Jacobians?","Fitfunction evaluations")
-	print(tbl)
-}
-
+tbl <- data.frame(
+	c("Yes","Yes","No","No"),c("No","Yes","Yes","No"),
+	c(powellrun1$output$evaluations,powellrun2$output$evaluations,powellrun6$output$evaluations,
+		powellrun7$output$evaluations), row.names=c("powellrun1","powellrun2","powellrun6","powellrun7"),
+	stringsAsFactors=F
+	)
+colnames(tbl) <- c("Gradient?","Jacobians?","Fitfunction evaluations")
+print(tbl)
 
 if(mxOption(NULL,"Default optimizer") == "SLSQP"){
 	#With GDsearch, Nelder-Mead can get a good solution even though none of its initial vertices is feasible,
