@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2017 The OpenMx Project
+#   Copyright 2007-2018 The OpenMx Project
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -109,11 +109,11 @@ set.seed(42)
 biRegModelRawBoot <- mxBootstrap(biRegModelRawOut, 10)
 omxCheckTrue(is.null(biRegModelRawBoot$output[["standardErrors"]]))
 bq1 <- summary(biRegModelRawBoot)[["bootstrapSE"]]
-omxCheckCloseEnough(cor(bq1, biRegModelRawOut$output$standardErrors), .91, .01)
+omxCheckCloseEnough(cor(bq1, biRegModelRawOut$output$standardErrors), .89, .01)
 
 biRegModelRawBoot <- mxBootstrap(biRegModelRawBoot)
 bq2 <- summary(biRegModelRawBoot)[["bootstrapSE"]]
-omxCheckCloseEnough(sum((bq2 - biRegModelRawOut$output$standardErrors)^2), 0, 1e-3)
+omxCheckCloseEnough(sum((bq2 - biRegModelRawOut$output$standardErrors)^2), 0, 3.5e-3)
 
 set.seed(42)
 biRegModelRawBoot <- mxBootstrap(biRegModelRawBoot, 10)
@@ -129,8 +129,8 @@ omxCheckCloseEnough(repl3[,omit], biRegModelRawBoot3$compute$output$raw[,omit],
 biRegModelRaw3 <- mxModel(
   biRegModelRaw,
   mxData(observed=cbind(multiData1,
-                        weight=biRegModelRawBoot3$compute$output$weight[[1]]),
-         type="raw", weight = "weight"))
+                        freq=biRegModelRawBoot3$compute$output$frequency[[1]]),
+         type="raw", frequency = "freq"))
 
 biRegModelRaw3 <- mxRun(biRegModelRaw3)
 
