@@ -1,7 +1,7 @@
 #ifndef _expectationMultigroup_H_
 #define _expectationMultigroup_H_
 
-void initFitMultigroup(omxFitFunction *off);
+omxFitFunction *initFitMultigroup();
 void omxMultigroupAdd(omxFitFunction *oo, omxFitFunction *grp);
 void mgSetFreeVarGroup(omxFitFunction *oo, FreeVarGroup *fvg);
 
