@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007-2017 The OpenMx Project
+ *  Copyright 2007-2018 The OpenMx Project
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -93,7 +93,7 @@ class omxExpectation {					// An Expectation
 	virtual const Eigen::Map<DataColumnType> getDataColumns();
 	virtual std::vector< omxThresholdColumn > &getThresholdInfo();
 
-	void loadThresholds(int numCols, int *thresholdColumn, int *thresholdNumber);
+	void loadThresholds();
 };
 
 omxExpectation *
