@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007-2017 The OpenMx Project
+ *  Copyright 2007-2018 The OpenMx Project
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -33,7 +33,7 @@ void omxState::omxProcessMxDataEntities(SEXP data, SEXP defvars)
 	SEXP nextLoc;
 	if(OMX_DEBUG) { mxLog("Processing %d data source(s).", Rf_length(data));}
 
-	SEXP listNames = Rf_getAttrib(data, R_NamesSymbol);
+	ProtectedSEXP listNames(Rf_getAttrib(data, R_NamesSymbol));
 
 	for(int index = 0; index < Rf_length(data); index++) {
 		ScopedProtect p1(nextLoc, VECTOR_ELT(data, index));			// Retrieve the data object
@@ -420,9 +420,16 @@ void omxState::omxProcessConstraints(SEXP constraints, FitContext *fc)
 		Rf_protect(nextLoc = VECTOR_ELT(nextVar, 1));
 		omxMatrix *arg2 = omxMatrixLookupFromState1(nextLoc, this);
 		Rf_protect(nextLoc = VECTOR_ELT(nextVar, 3));
+		const char *cname = CHAR(Rf_asChar(STRING_ELT(names, ci)));
+		if (arg1->dependsOnDefinitionVariables() ||
+		    arg2->dependsOnDefinitionVariables()) {
+			Rf_warning("Constraint '%s' depends on definition variables;"
+				   " This may not do what you expect. See ?mxConstraint",
+				   cname);
+		}
 		omxMatrix *jac = omxMatrixLookupFromState1(nextLoc, this);
 		int lin = INTEGER(VECTOR_ELT(nextVar,4))[0];
-		omxConstraint *constr = new UserConstraint(fc, CHAR(Rf_asChar(STRING_ELT(names, ci))), arg1, arg2, jac, lin);
+		omxConstraint *constr = new UserConstraint(fc, cname, arg1, arg2, jac, lin);
 		constr->opCode = (omxConstraint::Type) Rf_asInteger(VECTOR_ELT(nextVar, 2));
 		if (OMX_DEBUG) mxLog("constraint '%s' is type %d", constr->name, constr->opCode);
 		constr->prep(fc);
